/*
 * Diagram.java --- Class to draw a diagram.
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import javax.swing.JPanel;

import wassong.gui.tools.MyPropertiesListener;
import wassong.gui.tools.Resources;


public class Diagram extends JPanel implements MyPropertiesListener {

    //--------------------------------------------------------------------------
    // Constructors
    //--------------------------------------------------------------------------

    public Diagram( Resources		theResources,
		    MyProperties	theProperties,
		    ChessGame		aChessGame ) {

	super( new BorderLayout() );

	resources  = theResources;
	properties = theProperties;
	properties.addPropertyListener(
	    this, MyProperties.NOTATION_SHOW_SINGLE_MOVES );

	if ( IMAGES[ 0 ] == null ) {
	    Class	c       = this.getClass();
	    IMAGES[  0 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wKing32.gif" ) );
	    IMAGES[  1 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wPawn32.gif" ) );
	    IMAGES[  2 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wQueen32.gif" ) );
	    IMAGES[  3 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wRook32.gif" ) );
	    IMAGES[  4 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wBishop32.gif" ) );
	    IMAGES[  5 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/wKnight32.gif" ) );
	    IMAGES[  6 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bKing32.gif" ) );
	    IMAGES[  7 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bPawn32.gif" ) );
	    IMAGES[  8 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bQueen32.gif" ) );
	    IMAGES[  9 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bRook32.gif" ) );
	    IMAGES[ 10 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bBishop32.gif" ) );
	    IMAGES[ 11 ] = DEFAULT_TOOLKIT.getImage(
		c.getResource( "resources/bKnight32.gif" ) );

	    if ( HOTSPOT.getX() < 16 ) {
		 CURSOR_IMAGES[  0 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wKing16.gif" ) );
		 CURSOR_IMAGES[  1 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wPawn16.gif" ) );
		 CURSOR_IMAGES[  2 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wQueen16.gif" ) );
		 CURSOR_IMAGES[  3 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wRook16.gif" ) );
		 CURSOR_IMAGES[  4 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wBishop16.gif" ) );
		 CURSOR_IMAGES[  5 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/wKnight16.gif" ) );
		 CURSOR_IMAGES[  6 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bKing16.gif" ) );
		 CURSOR_IMAGES[  7 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bPawn16.gif" ) );
		 CURSOR_IMAGES[  8 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bQueen16.gif" ) );
		 CURSOR_IMAGES[  9 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bRook16.gif" ) );
		 CURSOR_IMAGES[ 10 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bBishop16.gif" ) );
		 CURSOR_IMAGES[ 11 ] = DEFAULT_TOOLKIT.getImage(
		     c.getResource( "resources/bKnight16.gif" ) );
	    }
	}

	chessGame = aChessGame;

	int[][]		chessboard = position().chessboard();
	Dimension	d	   = new Dimension(
	    ( chessboard.length + 1 ) * FONT_SIZE + 160,
	    ( chessboard[0].length + 2 ) * FONT_SIZE + 5 );
	this.setPreferredSize( d );

	this.setBackground( Color.white );

//	setBorder( BorderFactory.createEtchedBorder() );
//	Insets	insets = getBorder().getBorderInsets( this );
//	inset_top  = insets.top	 ;
//	inset_left = insets.left ;
	this.insetTop  = 0;
	this.insetLeft = 0;
    }

    //--------------------------------------------------------------------------
    // Paint functions
    //--------------------------------------------------------------------------

    private void drawChessboard( Graphics2D	g2d,
				 Position	position,
				 int		nbMoves,
				 int		dx,
				 int		dy ) {

	Color	blackSquareColor = new Color( 192, 192, 192 );

	int[][]	chessboard = position.chessboard();
	for ( int j = 0; j < chessboard[0].length; j++ ) {

	    g2d.drawString(
		new Character( (char)( '1' + j ) ).toString(),
		dx - DX_ROW_NUMBERS ,
		dy + DY_ROW_NUMBERS + (
		    chessboard[0].length - ( j + 1 ) ) * INCY_ROW_NUMBERS );

	    for ( int i = 0; i < chessboard.length; i++ ) {
		int	p = chessboard[ i ][ j ];
		boolean	isBlackSquare = ( ( i + j ) % 2 ) == 0;
		if ( isBlackSquare ) {
		    g2d.setColor( blackSquareColor );
		    g2d.fillRect(
			dx + i * FONT_SIZE ,
			dy + ( chessboard[0].length - j - 1 )
			* FONT_SIZE ,
			FONT_SIZE, FONT_SIZE );
		    g2d.setColor( Color.black );
		}
		if ( hidenSquareX == i && hidenSquareY == j ) {
		    continue;
		}
		if ( p != -1 ) {
		    Piece	pi    = position.piece( p );
		    int		index = pi.type();
		    if ( pi.colour() == Piece.BLACK ) {
			index += 6;
		    }
		    g2d.drawImage(
			IMAGES[ index ],
			dx + i * FONT_SIZE ,
			dy + ( chessboard[0].length - j - 1 ) * FONT_SIZE ,
			this );
		}
	    }
	}

	// Borders
	g2d.fillRect( dx, dy - 2, chessboard.length * FONT_SIZE, 2 );
	g2d.fillRect( dx,
		      dy + chessboard[ 0 ].length * FONT_SIZE,
		      chessboard.length * FONT_SIZE,
		      2 );
	if ( position.board().hasVerticalBorders() ) {
	    g2d.fillRect( dx - 2,
			  dy - 2,
			  2,
			  chessboard[ 0 ].length * FONT_SIZE + 4 );
	    g2d.fillRect( dx + chessboard.length * FONT_SIZE,
			  dy - 2,
			  2,
			  chessboard[ 0 ].length * FONT_SIZE + 4 );
	}

	for ( int i = 0; i < chessboard.length; i++ ) {
	    g2d.drawString(
		new Character( (char)( 'a' + i ) ).toString(),
		dx + DX_COLUMN_LETTERS + i * INCX_COLUMN_LETTERS ,
		dy + DY_COLUMN_LETTERS );
	}

	g2d.drawString( chessGame.nbMovesToString( nbMoves ),
			DX_DIAGRAM - DX_ROW_NUMBERS ,
			DY_COLUMN_LETTERS + FONT_SIZE );
	g2d.drawString( "(" + position.numberOfWhitePieces()
			+ " + " + position.numberOfBlackPieces() + ')'
			+ "   "
			+ resources.getString( position.conditionName() ),
			DX_DIAGRAM + 6 * FONT_SIZE  ,
			DY_COLUMN_LETTERS + FONT_SIZE );
    }

    public void paintComponent( Graphics g ) {

	super.paintComponent( g );

	Graphics2D	g2d = (Graphics2D)g;
	g2d.setFont( NORMAL_FONT );

	drawChessboard( g2d             ,
			position()      ,
			numberOfMoves() ,
			DX_DIAGRAM      ,
			DY_DIAGRAM      );
    }

    //--------------------------------------------------------------------------
    // Overloadable functions
    //--------------------------------------------------------------------------

    public Position position() {
	return chessGame.finalPosition();
    }

    public int numberOfMoves() {
	return chessGame.finalNumberOfMoves();
    }

    //--------------------------------------------------------------------------
    // Other functions
    //--------------------------------------------------------------------------
    public final ChessGame chessGame() {
	return this.chessGame;
    }

    //--------------------------------------------------------------------------
    // Other functions
    //--------------------------------------------------------------------------

    protected void setTheCursor( int type ) {

	final String[] cursorNames = {
	    "White King"  ,
	    "White Pawn"  ,
	    "White Queen" ,
	    "White Rook"  ,
	    "White Bishop",
	    "White Knight",
	    "Black King"  ,
	    "Black Pawn"  ,
	    "Black Queen" ,
	    "Black Rook"  ,
	    "Black Bishop",
	    "Black Knight"
	};

	if ( type < 0 || type >= cursorNames.length ) {
	    this.setCursor( Cursor.getDefaultCursor() );
	    chessCursorType = -1;
	} else {
	    this.setCursor(
		DEFAULT_TOOLKIT.createCustomCursor(
		    HOTSPOT.getX() < 16 ? CURSOR_IMAGES[type] : IMAGES[type],
		    HOTSPOT,
		    cursorNames[ type ] ) );
	    chessCursorType = type;
	}
    }

    protected void hideSquare( int x, int y ) {
	hidenSquareX = x;
	hidenSquareY = y;
    }

    //--------------------------------------------------------------------------
    // Other events
    //--------------------------------------------------------------------------

    public void chessGameChanged() {
	repaint();
    }

    public void chessPositionChanged() {
	repaint();
    }

    public void propertyChanged( String property ) {
	repaint();
    }

    //--------------------------------------------------------------------------
    // Access to protected attributes
    //--------------------------------------------------------------------------

    protected int insetTop() {
	return this.insetTop;
    }

    protected int insetLeft() {
	return this.insetLeft;
    }

    protected int chessCursorType() {
	return this.chessCursorType;
    }

    //--------------------------------------------------------------------------
    // Attributes
    //--------------------------------------------------------------------------

    protected Resources		resources;
    protected MyProperties	properties;

    private final ChessGame	chessGame;

    private final int		insetTop;
    private final int		insetLeft;

    private int			chessCursorType = -1;

    private int			hidenSquareX	= -1;
    private int			hidenSquareY	= -1;

    protected static final int	FONT_SIZE   = 32;
    protected static final Font	NORMAL_FONT = new Font(
	"Arial", Font.PLAIN, FONT_SIZE / 2 );

    protected static final int	DX_DIAGRAM = FONT_SIZE;
    protected static final int	DY_DIAGRAM = 7;

    private static final int	DX_ROW_NUMBERS     = FONT_SIZE - FONT_SIZE / 3;
    private static final int	DY_ROW_NUMBERS     = FONT_SIZE - 9;
    private static final int	INCY_ROW_NUMBERS   = FONT_SIZE;
    private static final int	DX_COLUMN_LETTERS  = FONT_SIZE / 3;
    private static final int	DY_COLUMN_LETTERS  = 9 * FONT_SIZE - 7;
    private static final int	INCX_COLUMN_LETTERS = FONT_SIZE;

    private static final Toolkit	DEFAULT_TOOLKIT;

    // For custom cursors
    protected static final Image[]	IMAGES = new Image[ 12 ];
    private static final Point		HOTSPOT;
    private static final Image[]	CURSOR_IMAGES;


    static {
	DEFAULT_TOOLKIT = Toolkit.getDefaultToolkit();
	Dimension d = DEFAULT_TOOLKIT.getBestCursorSize( FONT_SIZE,
							 FONT_SIZE );
	HOTSPOT = new Point( (int)d.getWidth()  / 2,
			     (int)d.getHeight() / 2 );
	if ( HOTSPOT.getX() >= 16 ) {
	    CURSOR_IMAGES = IMAGES;
	} else {
	    CURSOR_IMAGES = new Image[ 12 ];
	}
    }
}
