/*
 * DiagramPanel.java --- Class to draw the diagrams.
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import wassong.gui.tools.MyPropertiesListener;


public class DiagramPanel extends JPanel
    implements ChessGameListener, MyPropertiesListener {

    //--------------------------------------------------------------------------
    // Constructors
    //--------------------------------------------------------------------------

    public DiagramPanel( Diagram aDiagram, MyProperties properties ) {

	super( new BorderLayout() );

	diagram = aDiagram;

	setBorder( BorderFactory.createEtchedBorder() );
	Insets	insets = getBorder().getBorderInsets( this );
	insetTop  = insets.top;
	insetLeft = insets.left;

	add( diagram, BorderLayout.CENTER );

	diagramForsythe = new JLabel( diagram.position().toString() );
	diagramForsythe.setBorder( BorderFactory.createEtchedBorder() );
	diagramForsythe.setHorizontalAlignment( SwingConstants.CENTER );
	diagramForsythe.setMinimumSize( new Dimension( 0, 0 ) );
	add( diagramForsythe, BorderLayout.SOUTH );

	diagram.chessGame().addChessPositionListener( this );

	properties.addPropertyListener( this, MyProperties.OUTPUT_LANGUAGE );
    }

    //--------------------------------------------------------------------------
    // Other events
    //--------------------------------------------------------------------------

    public void chessGameChanged() {
	chessPositionChanged();
    }

    public void chessPositionChanged() {
	diagramForsythe.setText( diagram.position().toString() );
	diagram.repaint();
    }

    public void propertyChanged( String property ) {
	// There may be a problem of order of call of the PropertiesListener :
	// if the this call is made before the call for PieceProperties, the
	// pieces are still in the old language so the following is no YET
	// correct.  A test shows that this does not happen but...
	diagramForsythe.setText( diagram.position().toString() );
    }

    //--------------------------------------------------------------------------
    // Attributes
    //--------------------------------------------------------------------------

    private final Diagram	diagram;
    private final JLabel	diagramForsythe;

    private final int		insetTop;
    private final int		insetLeft;
}
