/*
 * LanguageCombo.java --- Combo box to choose the language.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.StringTokenizer;
import javax.swing.JComboBox;

import wassong.gui.tools.MyPropertiesListener;
import wassong.gui.tools.Resources;


public class LanguageCombo extends JComboBox implements MyPropertiesListener {

    public LanguageCombo( Resources		theResources,
			  final MyProperties	theProperties ) {
	resources  = theResources;
	properties = theProperties;

	StringTokenizer	t = new StringTokenizer(
	    resources.getString( OUTPUT_LANGUAGES_LIST ), "," );
	while ( t.hasMoreTokens() ) {
	    String s = t.nextToken().trim();
	    addItem( s );
	}

	properties.addPropertyListener(
	    this, MyProperties.OUTPUT_LANGUAGE );

	setIndex();
	setMaximumSize( getMinimumSize() );

	addActionListener( new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
		    String newLanguage = (String)getSelectedItem();
		    properties.setProperty(
			MyProperties.OUTPUT_LANGUAGE,
			newLanguage.substring( 0,
					       newLanguage.indexOf( ' ' ) ) );
		}
	    } );
    }

    public void propertyChanged( String property ) {
	setIndex();
    }

    //--------------------------------------------------------------------------

    private void setIndex() {
	StringTokenizer	t = new StringTokenizer(
	    resources.getString( OUTPUT_LANGUAGES_LIST ), "," );
	int	counter = 0;
	String	language
	    = properties.getProperty( MyProperties.OUTPUT_LANGUAGE );
	while ( t.hasMoreTokens() ) {
	    String s = t.nextToken().trim();
	    if ( s.startsWith( language ) ) {
		setSelectedIndex( counter );
		return;
	    }
	    counter++;
	}
    }

    //--------------------------------------------------------------------------

    private Resources		resources;
    private MyProperties	properties;

    private static final String OUTPUT_LANGUAGES_LIST = "outputLanguages";
}
