/*
 * MoveProperties.java --- Update move properties according to user choices.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import wassong.gui.tools.MyPropertiesListener;
import wassong.gui.tools.Resources;

public class MoveProperties implements MyPropertiesListener {

    public MoveProperties( Resources	theResources,
			   MyProperties	theProperties ) {
	resources  = theResources;
	properties = theProperties;

	properties.addPropertyListener(
	    this, MyProperties.NOTATION_USE_SPECIAL_CHARACTERS );
	properties.addPropertyListener(
	    this, MyProperties.NOTATION_CASTLING_USE_ZEROS );

	updateSpecialNotation();
	updateCastlingNotation();
    }

    public void propertyChanged( String property ) {
	if ( property.equals( MyProperties.NOTATION_USE_SPECIAL_CHARACTERS ) ) {
	    updateSpecialNotation();
	} else {
	    updateCastlingNotation();
	}
    }

    //--------------------------------------------------------------------------

    private void updateSpecialNotation() {
	Move.setSpecialNotation(
	    properties.getBooleanProperty(
		MyProperties.NOTATION_USE_SPECIAL_CHARACTERS ) );
    }

    private void updateCastlingNotation() {
	Move.setCastlingNotationUseZeroes(
	    properties.getBooleanProperty(
		MyProperties.NOTATION_CASTLING_USE_ZEROS ) );
    }

    //--------------------------------------------------------------------------

    private Resources		resources;
    private MyProperties	properties;

}
