/*
 * Natch.java --- Execute Natch.
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;


public final class Natch {

    public static String executableName() {
	return EXECUTABLE_NAME;
    }

    public static List execute( MyProperties	properties,
				Position	position,
				int		numberOfMoves ) {

	FileWriter	out;
	try {
	    out = new FileWriter( INPUT_FILE );
	} catch ( IOException ex ) {
	    Error.errorMessage(
		"save-error-title",
		"unable-to-create",
		INPUT_FILE );
	    return null;
	}

	try {
	    out.write( position.toString() + '\n' + numberOfMoves );
	} catch ( IOException ioe ) {
	    Error.errorMessage(
		"save-error-title",
		"error-during-save",
		INPUT_FILE );
	} finally {
	    try {
		out.close();
	    } catch ( IOException ioe ) {
		// Nothing to do
	    }
	}

	String[]	envp = new String[ 1 ];
	envp[ 0 ] = "NATCH=";

	String	path = properties.getProperty( MyProperties.NATCH_PATH );
	if ( path.length() > 0 && !path.endsWith( File.separator ) ) {
	    path += File.separatorChar;
	}

	String	args = properties.getProperty( MyProperties.NATCH_PARAMETERS );

	// Remove the result file if it exists
	File f = new File( RESULT_FILE );
	if ( f.exists() ) {
	    f.delete();
	}

	try {
	    Process p = Runtime.getRuntime().exec(
		path + executableName() + ' ' + args
		+ " -o " + RESULT_FILE + ' ' + INPUT_FILE,
		envp );
	    p.waitFor();

	    try {
		Reader		r  = new FileReader( new File( RESULT_FILE ) );
		BufferedReader	in = new BufferedReader( r );

		String	s;
		List	comments   = new ArrayList( 100 );
		int	nbMaxLines = 100;
		while ( ( s = in.readLine() ) != null && nbMaxLines > 0 ) {
		    comments.add( s );
		    nbMaxLines--;
		}
		return comments;
	    } catch ( FileNotFoundException fnfe ) {
		Error.errorMessage(
		    "Natch-error-title",
		    "Natch-result-file-inexistant",
		    "" );
	    }
	} catch ( IOException ioe ) {
	    Error.errorMessage(
		"Natch-error-title",
		"unable-to-execute-Natch",
		path + executableName() );
	} catch ( InterruptedException ie ) {
	    Error.errorMessage(
		"Natch-error-title",
		"Natch-execution-interrupted",
		"" );
	}
	return null;
    }

    //--------------------------------------------------------------------------

    private Natch() { } // Avoid instantiation

    //--------------------------------------------------------------------------

    private static final String		EXECUTABLE_NAME ;
    private static final String		RESULT_FILE = "iNatch.res" ;
    private static final String		INPUT_FILE  = "iNatch.txt" ;

    static {
	String	osName = System.getProperty( "os.name" );
	EXECUTABLE_NAME = osName.startsWith( "Win" ) ? "Natch.exe" : "Natch";
    }
}
