/*
 * PieceProperties.java --- Update piece properties according to user choices.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.util.StringTokenizer;

import wassong.gui.tools.MyPropertiesListener;
import wassong.gui.tools.Resources;

public class PieceProperties implements MyPropertiesListener {

    public PieceProperties( Resources		theResources,
			    MyProperties	theProperties ) {
	resources  = theResources;
	properties = theProperties;

	properties.addPropertyListener( this, MyProperties.OUTPUT_LANGUAGE );
	properties.addPropertyListener(
	    this, MyProperties.NOTATION_USE_FRENCH_C_AND_E );

	updatePieceInitials();
	updateColumnNames();
    }

    public void propertyChanged( String property ) {
	if ( property.equals( MyProperties.OUTPUT_LANGUAGE ) ) {
	    updatePieceInitials();
	}
	updateColumnNames();
    }

    //--------------------------------------------------------------------------

    private void updatePieceInitials() {

	String	pieceInitials;

	StringTokenizer	t = new StringTokenizer(
	    resources.getString( "outputLanguages" ), "," );
	int	currentChoice = 0;
	int	counter       = 0;
	String	language
	    = properties.getProperty( MyProperties.OUTPUT_LANGUAGE );
	while ( t.hasMoreTokens() ) {
	    String s = t.nextToken().trim();
	    if ( s.startsWith( language ) ) {
		currentChoice = counter;
		break;
	    }
	    counter++;
	}
	t = new StringTokenizer( resources.getString( "forsytheInitials" ) );
	do {
	    pieceInitials = t.nextToken();
	    currentChoice--;
	} while ( currentChoice >= 0 );

	Piece.setInitials( pieceInitials, "wb" );
    }

    private void updateColumnNames() {
	Piece.setColumnsFrenchNames(
	    properties.getBooleanProperty(
		MyProperties.NOTATION_USE_FRENCH_C_AND_E )
	    &&	( !properties.getBooleanProperty(
		      MyProperties.NOTATION_USE_FRENCH_C_AND_E_IF_FRENCH )
		  || properties.getProperty( MyProperties.OUTPUT_LANGUAGE )
		  .equals( "Fran\u00E7ais" ) ) );
    }

    //--------------------------------------------------------------------------

    private Resources		resources;
    private MyProperties	properties;
}
