/*
 * PositionFactory.java --- Create `Position' from `String'.
 *
 * Copyright (c) 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.util.StringTokenizer;


/**
 * @invariant ( status() != CORRECT_MOVE ) || ( move() != null )
 */
public final class PositionFactory {

    public static final int	CORRECT_POSITION = 0;
    public static final int	ILLEGAL_POSITION = 1;

    public static int status() {
	return status;
    }

    public static Position position() {
	return position;
    }

    /**
     * Create Position from String.  Creation status can be checked via
     * `status'.  Created position can be retrieved via `position'.
     */
    public static void make( Board board, Condition condition, String s ) {

	s = s.trim();

	StringTokenizer t = new StringTokenizer( forsytheInitials );

	while ( t.hasMoreTokens() ) {
	    String currentInitials = t.nextToken();

	    // Verify that all the characters are correct for any possible
	    // language.  Verify also that they are exactly 2 kings.
	    String	authorisedChars = currentInitials + "12345678/";
	    int		nbKings = 0;
	    boolean	incorrectCharacter = false;
	    for ( int index = 0;
		  !incorrectCharacter && index < s.length();
		  index++ ) {
		char c = Character.toLowerCase( s.charAt( index ) );
		if ( c == authorisedChars.charAt( 0 ) ) {
		    nbKings++;
		}
		incorrectCharacter = ( authorisedChars.indexOf( c ) == -1 );
	    }
	    if ( incorrectCharacter || nbKings != 2 ) {
		continue;
	    }

	    position = new Position( board, condition, false );

	    int	row    = board.sizeY() - 1;
	    int	column = 0;
	    for ( int index = 0; index < s.length(); index++ ) {
		char	c = s.charAt( index );
		if ( c >= '1' && c <= '0' + board.sizeX() ) {
		    column += c - '0';
		} else if ( c == '/' ) {
		    row--;
		    column = 0;
		} else if ( column < board.sizeX() ) {
		    int[] types = {
			Piece.KING,
			Piece.PAWN,
			Piece.QUEEN,
			Piece.ROOK,
			Piece.BISHOP,
			Piece.KNIGHT,
			Piece.KNIGHT
		    };
		    int colour = Character.isUpperCase( c )
			? Piece.WHITE : Piece.BLACK;
		    c = Character.toLowerCase( c );
		    int pieceIndex = 0;
		    while ( c != currentInitials.charAt( pieceIndex ) ) {
			pieceIndex++;
		    }
		    position.addPieceAt( new Piece(
					     types[ pieceIndex ],
					     colour,
					     column,
					     row ) );
		    column++;
		}
	    }
	    status = CORRECT_POSITION;
	    return;
	}
	status = ILLEGAL_POSITION;
    }

    //--------------------------------------------------------------------------

    public static void setForsytheInitials( String initials ) {
	if ( initials == null ) {
	    forsytheInitials = DEFAULT_FORSYTHE_INITIALS;
	} else {
	    forsytheInitials = initials;
	}
    }

    //--------------------------------------------------------------------------

    private PositionFactory() { } // Avoid instantiation

    //--------------------------------------------------------------------------

    private static final String DEFAULT_FORSYTHE_INITIALS = "kpqrbns";
    private static String	forsytheInitials = DEFAULT_FORSYTHE_INITIALS;

    private static Position	position = null;
    private static int		status	 = ILLEGAL_POSITION;

}
