/*
 * ToolBar.java --- Create a toolbar from the resources.
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch;

import java.awt.Component;
import java.awt.Insets;
import java.util.Map;
import java.util.MissingResourceException;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JToolBar;

import wassong.gui.tools.Resources;


public class ToolBar {

    /**
     * Construct a toolbar from a resource file, a menu name and a
     * command table.
     */
    public ToolBar(
	String		title,
	String		toolBarName,
	Map		theCommands,
	Component[]	components,
	Resources	resources ) {

	commands = theCommands;

	toolbar = new JToolBar( title + ' ' + toolBarName );
	String[] toolKeys = resources.getTokenizedString( toolBarName );
	for ( int i = 0; i < toolKeys.length; i++ ) {
	    if ( toolKeys[i].equals( "-" ) ) {
		toolbar.addSeparator();
	    } else {
		toolbar.add( createTool( toolKeys[ i ], resources ) );
	    }
	}

	if ( components != null ) {
	    for ( int i = 0; i < components.length; i++ ) {
		toolbar.addSeparator();
		toolbar.add( components[ i ] );
	    }
	}
	toolbar.add( Box.createHorizontalGlue() );
    }

    /**
     * Return the toolbar created
     */
    public JToolBar getToolBar() {
	return toolbar;
    }

    /**
     * Hook through which every toolbar item is created.
     */
    private Component createTool( String key, Resources resources ) {
	return createToolbarButton( key, resources );
    }

    /**
     * Create a button to go inside of the toolbar.	 By default this
     * will load an image resource.	 The image filename is relative to
     * the classpath (including the '.' directory if its a part of the
     * classpath), and may either be in a JAR file or a separate file.
     *
     * @param key The key in the resource file to serve as the basis
     *	of lookups.
     */
    private JButton createToolbarButton( String key, Resources resources ) {
	JButton b = new JButton() {
		public float getAlignmentY() { return 0.5f; }
	    };
	b.setRequestFocusEnabled( false );
	b.setMargin( new Insets( 1, 1, 1, 1 ) );

	Action a = getAction( key );
	if ( a != null ) {
	    b.setAction( a );
	    if ( a.getValue( Action.SMALL_ICON ) != null ) {
		b.setText( "" );
	    }
	} else {
	    b.setEnabled( false );
	}

	try {
	    b.setToolTipText( resources.getString(
				  key + resources.TIP_SUFFIX ) );
	} catch ( MissingResourceException e ) {
	    // Nothing to do
	}

	return b;
    }

    protected Action getAction( String cmd ) {
	return (Action)commands.get( cmd );
    }

    private JToolBar	toolbar;
    private Map		commands;

}
