/*
 * MoveTest.java --- JUnit tests for Move class.
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch.tests;


import java.io.BufferedReader;
import java.io.FileReader;
import java.util.StringTokenizer;

import iNatch.Move;
import iNatch.MoveFactory;
import iNatch.OrthodoxCondition;
import iNatch.Piece;
import iNatch.Position;
import iNatch.PositionFactory;
import iNatch.StandardBoard;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class MoveTest extends TestCase {

    public MoveTest( String name ) {
	super( name );
    }

    public static Test suite() {
	return new TestSuite( MoveTest.class );
    }

    public static void main( String[] args ) {
	junit.textui.TestRunner.run ( suite() );
    }

    //-------------------------------------------------------------------------

    public void testConstructor() throws Exception {
	BufferedReader	in = new BufferedReader(
	    new FileReader( "iNatch/tests/testMove.txt" ) );

	int line = 1;
	String s = in.readLine();
	if ( s == null ) {
	    fail( "File testMove.txt is empty" );
	    return;
	}
	PositionFactory.make( new StandardBoard(),
			      new OrthodoxCondition(),
			      s );
	if ( PositionFactory.status() != PositionFactory.CORRECT_POSITION ) {
	    fail( "Wrong position in file testMove.txt" );
	    return;
	}
	Position position = PositionFactory.position();

	while ( ( s = in.readLine() ) != null ) {
	    line++;
	    StringTokenizer	t = new StringTokenizer( s );
	    String		test     = null;
	    String		expected = null;
	    if ( t.hasMoreTokens() ) {
		test = t.nextToken();
		if ( t.hasMoreTokens() ) {
		    expected = t.nextToken();
		}
	    }
	    if ( expected == null ) {
		fail( "Incorrect line " + line
		      + " in testMove.txt : " + s );
		continue;
	    }

	    String result;
	    MoveFactory.make( test,
			      Piece.WHITE,
			      "kpqrbns",
			      position );
	    int status = MoveFactory.status();
	    if ( status == MoveFactory.CORRECT_MOVE ) {
		result = MoveFactory.move().toString();
	    } else if ( status == MoveFactory.EMPTY_MOVE ) {
		result = "Empty";
	    } else {
		result = "Illegal";
	    }
	    assertEquals( "Line " + line + " of testMove.txt ("
			  + test + ") : ",
			  expected, result );
	}
    }

    public void testToString() throws Exception {

	PositionFactory.make( new StandardBoard(),
			      new OrthodoxCondition(),
			      "1QQ5/Q7/8/1Q6/8/8/8/k1K" );
	if ( PositionFactory.status() != PositionFactory.CORRECT_POSITION ) {
	    fail( "Wrong position" );
	    return;
	}
	Position position = PositionFactory.position();

	assertEquals( "wKe1-wKe2", m1.toString() );
	assertEquals( "bNg8-wPf6", m2.toString() );

	Piece p1 = new Piece( Piece.QUEEN, Piece.WHITE, "a8" );
	Piece p2 = new Piece( Piece.QUEEN, Piece.WHITE, "b8" );
	Piece p3 = new Piece( Piece.QUEEN, Piece.WHITE, "b6" );
	Piece p4 = new Piece( Piece.QUEEN, Piece.WHITE, "b7" );

	assertEquals( "Qba8",  new Move( p2, p1 ).toShortString( position ) );
	assertEquals( "Q8b6",  new Move( p2, p3 ).toShortString( position ) );
	assertEquals( "Qb8b7", new Move( p2, p4 ).toShortString( position ) );
    }

    public void testEquals() {
	assertTrue  ( !m1.equals( null ) );
	assertEquals( m1, m1 );
	assertEquals( m1,
		      new Move( new Piece( Piece.KING, Piece.WHITE, "e1" ),
				new Piece( Piece.KING, Piece.WHITE, "e2" ) ) );
	assertTrue  ( !m1.equals( m2 ) );
    }

    //-------------------------------------------------------------------------

    protected void setUp() {
	m1 = new Move( new Piece( Piece.KING,   Piece.WHITE, "e1" ),
		       new Piece( Piece.KING,   Piece.WHITE, "e2" ) );
	m2 = new Move( new Piece( Piece.KNIGHT, Piece.BLACK, "g8" ),
		       new Piece( Piece.PAWN,   Piece.WHITE, "f6" ) );
    }

    protected void tearDown() {
	m1 = null;
	m2 = null;
    }

    protected Move	m1;
    protected Move	m2;
}
