/*
 * PieceTest.java --- Unit test (using the JUnit framework) for the Piece class
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch.tests;

import iNatch.Piece;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class PieceTest extends TestCase {

    public PieceTest( String name ) {
	super( name );
    }

    public static Test suite() {
	return new TestSuite( PieceTest.class );
    }

    public static void main( String[] args ) {
	junit.textui.TestRunner.run ( suite() );
    }

    //-------------------------------------------------------------------------

    public void testStringConstructor() {

	// Uppercase character is allowed
	Piece p = new Piece( Piece.QUEEN, Piece.WHITE, "A1" );

	// Out of bound squares should be illegal
	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, "a0" );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, "a9" );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, "@1" );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, "i1" );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}
    }

    public void testNumericConstructor() {

	Piece p;

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, 0, -1 );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, 0, 8 );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, -1, 1 );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}

	try {
	    p = new Piece( Piece.QUEEN, Piece.WHITE, 8, 1 );
	    fail( "Should throw IllegalArgumentException" );
	} catch ( IllegalArgumentException iae ) {
	    // Nothing to do
	}
    }

    public void testSelectors() {
	assertTrue( p1.type()          == Piece.KING
		    && p1.typeChar()   == 'K'
		    && p1.colour()     == Piece.WHITE
		    && p1.row()        == 0
		    && p1.rowChar()    == '1'
		    && p1.column()     == 4
		    && p1.columnChar() == 'e' );
    }

    public void testToString() {
	Piece.setInitials( "rpdtfc", "bn" );
	Piece.setColumnsFrenchNames( true );
	assertEquals( "bR1", p1.toString() );
	assertEquals( "nC8", p2.toString() );

	Piece.setInitials( null, null );
	Piece.setColumnsFrenchNames( false );
	assertEquals( "wKe1", p1.toString() );
	assertEquals( "bNc8", p2.toString() );
    }

    public void testEquals() {
	assertTrue  ( !p1.equals( null ) );
	assertEquals( p1, p1 );
	assertEquals( p1, new Piece( Piece.KING, Piece.WHITE, "e1" ) );
	assertTrue  ( !p1.equals( p2 ) );
    }

    //-------------------------------------------------------------------------

    protected void setUp() {
	p1 = new Piece( Piece.KING,   Piece.WHITE, "e1" );
	p2 = new Piece( Piece.KNIGHT, Piece.BLACK, "c8" );
    }

    protected void tearDown() {
	p1 = null;
	p2 = null;
    }

    protected Piece	p1;
    protected Piece	p2;
}
