/*
 * SPGMoveTest.java --- JUnit tests for SPGMove class.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of iNatch.
 *
 * iNatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * iNatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package iNatch.tests;

import iNatch.Move;
import iNatch.OrthodoxCondition;
import iNatch.Piece;
import iNatch.Position;
import iNatch.PositionFactory;
import iNatch.SPGMove;
import iNatch.StandardBoard;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class SPGMoveTest extends TestCase {

    public SPGMoveTest( String name ) {
	super( name );
    }

    public static Test suite() {
	return new TestSuite( SPGMoveTest.class );
    }

    public static void main( String[] args ) {
	junit.textui.TestRunner.run ( suite() );
    }

    //-------------------------------------------------------------------------

    public void testOrderMoves() {
	PositionFactory.make( new StandardBoard(),
			      new OrthodoxCondition(),
			      "4k3/8/8/8/2B5/8/8/4K3" );
	if ( PositionFactory.status() != PositionFactory.CORRECT_POSITION ) {
	    fail( "Illegal position" );
	    return;
	}
	Position p = PositionFactory.position();

	Move		previousMove =
	    new Move( new Piece( Piece.KING,   Piece.BLACK, "e7" ),
		      new Piece( Piece.KING,   Piece.BLACK, "e8" ) );
	Move		otherPreviousMove =
	    new Move( new Piece( Piece.BISHOP, Piece.WHITE, "f1" ),
		      new Piece( Piece.BISHOP, Piece.WHITE, "c4" ) );
	Move[]		moves = new Move[ 3 ];

	Move bishopA2 =
	    new Move( new Piece( Piece.BISHOP, Piece.WHITE, "c4" ),
		      new Piece( Piece.BISHOP, Piece.WHITE, "a2" ) );
	Move bishopE2 =
	    new Move( new Piece( Piece.BISHOP, Piece.WHITE, "c4" ),
		      new Piece( Piece.BISHOP, Piece.WHITE, "e2" ) );
	Move bishopF1 =
	    new Move( new Piece( Piece.BISHOP, Piece.WHITE, "c4" ),
		      new Piece( Piece.BISHOP, Piece.WHITE, "f1" ) );
	Move kingF1 =
	    new Move( new Piece( Piece.KING,   Piece.WHITE, "e1" ),
		      new Piece( Piece.KING,   Piece.WHITE, "f1" ) );
	Move kingD1 =
	    new Move( new Piece( Piece.KING,   Piece.WHITE, "e1" ),
		      new Piece( Piece.KING,   Piece.WHITE, "d1" ) );
	Move kingE2 =
	    new Move( new Piece( Piece.KING,   Piece.WHITE, "e1" ),
		      new Piece( Piece.KING,   Piece.WHITE, "e2" ) );

	moves[ 0 ] = bishopA2;
	moves[ 1 ] = bishopF1;
	moves[ 2 ] = kingD1;

	SPGMove[]	orderedMoves =
	    SPGMove.orderMoves( moves, 3, p, previousMove, otherPreviousMove );

	assertEquals( "Wrong table length", moves.length, orderedMoves.length );
	assertEquals( bishopA2, (Move)orderedMoves[ 0 ] );
	assertEquals( kingD1,   (Move)orderedMoves[ 1 ] );
	assertEquals( bishopF1, (Move)orderedMoves[ 2 ] );
	assertEquals( SPGMove.MOVE_OK,  orderedMoves[ 0 ].status() );
	assertEquals( SPGMove.MOVE_OLD, orderedMoves[ 1 ].status() );
	assertEquals( SPGMove.MOVE_BAD, orderedMoves[ 2 ].status() );


	moves[ 0 ] = bishopE2;
	moves[ 1 ] = kingF1;
	moves[ 2 ] = kingE2;

	orderedMoves =
	    SPGMove.orderMoves( moves, 3, p, previousMove, otherPreviousMove );

	assertEquals( "Wrong table length", moves.length, orderedMoves.length );
	assertEquals( kingF1,   (Move)orderedMoves[ 0 ] );
	assertEquals( kingE2,   (Move)orderedMoves[ 1 ] );
	assertEquals( bishopE2, (Move)orderedMoves[ 2 ] );
	assertEquals( SPGMove.MOVE_OK,  orderedMoves[ 0 ].status() );
	assertEquals( SPGMove.MOVE_OK,  orderedMoves[ 1 ].status() );
	assertEquals( SPGMove.MOVE_BAD, orderedMoves[ 2 ].status() );

	orderedMoves =
	    SPGMove.orderMoves( moves, 3, p, null, otherPreviousMove );

	assertEquals( "Wrong table length", moves.length, orderedMoves.length );
	assertEquals( bishopE2, (Move)orderedMoves[ 0 ] );
	assertEquals( kingF1,   (Move)orderedMoves[ 1 ] );
	assertEquals( kingE2,   (Move)orderedMoves[ 2 ] );
	assertEquals( SPGMove.MOVE_OK, orderedMoves[ 0 ].status() );
	assertEquals( SPGMove.MOVE_OK, orderedMoves[ 1 ].status() );
	assertEquals( SPGMove.MOVE_OK, orderedMoves[ 2 ].status() );
    }
}
