/*
 * MacMenuItem.java --- Build a special menu item for Mac OS
 *
 * Copyright (c) 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.util.Map;
import javax.swing.Action;
import javax.swing.SwingUtilities;

import com.apple.mrj.MRJAboutHandler;
import com.apple.mrj.MRJApplicationUtils;
import com.apple.mrj.MRJPrefsHandler;
import com.apple.mrj.MRJQuitHandler;

final class MacMenuItem {

    static boolean isMacSpecific( String name ) {
	return name.equals( QUIT )
	    || name.equals( PREFERENCES )
	    || name.equals( ABOUT );
    }

    static void create( final String name, final Map actions ) {

	if ( name.equals( QUIT ) ) {
	    MRJApplicationUtils.registerQuitHandler( new MRJQuitHandler() {
		    public void handleQuit() {
			SwingUtilities.invokeLater( new Runnable() {
				public void run() {
				    Action a = (Action)actions.get( name );
				    a.actionPerformed( null );
				}
			    });
			throw new IllegalStateException(
			    "Give control back to the application" );
		    }
		});
	} else if ( name.equals( PREFERENCES ) ) {
	    MRJApplicationUtils.registerPrefsHandler( new MRJPrefsHandler() {
		    public void handlePrefs() {
			SwingUtilities.invokeLater( new Runnable() {
				public void run() {
				    Action a = (Action)actions.get( name );
				    a.actionPerformed( null );
				}
			    });
		    }
		});
	} else if ( name.equals( ABOUT ) ) {
	    MRJApplicationUtils.registerAboutHandler( new MRJAboutHandler() {
		    public void handleAbout() {
			SwingUtilities.invokeLater( new Runnable() {
				public void run() {
				    Action a = (Action)actions.get( name );
				    a.actionPerformed( null );
				}
			    });
		    }
		});
	}
    }

    //--------------------------------------------------------------------------

    private MacMenuItem() { } // Avoid instantiation

    //--------------------------------------------------------------------------

    private static final String	QUIT	    = "quit";
    private static final String	PREFERENCES = "preferences";
    private static final String	ABOUT	    = "about";
}
