/*
 * MyMenu.java --- Build a menu from the XML description.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.util.Enumeration;
import java.util.Map;
import java.util.MissingResourceException;
import javax.swing.JMenu;

import nanoxml.XMLElement;

public class MyMenu extends JMenu {
    MyMenu( XMLElement	xml,
	    Map		actions,
	    Map		radiobuttonGroups,
	    Resources	resources,
	    boolean	canShowMnemonics,
	    boolean	isMacAqua ) {

	super();

	String id = xml.getStringAttribute( "name", "?" ).trim();
	NameAndMnemonic nam;
	try {
	    nam =
		new NameAndMnemonic( resources.getString( MENUS_PREFIX + id ) );
	} catch ( MissingResourceException e ) {
	    nam = new NameAndMnemonic( id );
	}
	this.setText( nam.name() );
	if ( canShowMnemonics && nam.hasMnemonic() ) {
	    this.setMnemonic( nam.mnemonic() );
	}

	/* Separators are ignored at the beginning and at the end of a menu.
	 * Two consecutive separator create only one.  Except for errors in the
	 * menu description, this is also necessary on Mac, where 3 menu items
	 * are put in special places, so there may be too much separators.
	 */
	boolean isSeparatorRequired = false;
	boolean isMenuEmpty = true;

	Enumeration childs = xml.enumerateChildren();
	while ( childs.hasMoreElements() ) {
	    XMLElement child = (XMLElement)childs.nextElement();
	    String name = child.getName();
	    if ( name.equals( "menu" ) ) {
		if ( isSeparatorRequired ) {
		    this.addSeparator();
		    isSeparatorRequired = false;
		}
		this.add( new MyMenu( child,
				      actions,
				      radiobuttonGroups,
				      resources,
				      canShowMnemonics,
				      isMacAqua ) );
		isMenuEmpty = false;
	    } else if ( name.equals( "separator" ) ) {
		if ( !isMenuEmpty ) {
		    isSeparatorRequired = true;
		}
	    } else if ( name.equals( "menuitem" ) ) {
		id = child.getStringAttribute( "name", "?" ).trim();
		if ( isMacAqua && MacMenuItem.isMacSpecific( id ) ) {
		    MacMenuItem.create( id, actions );
		} else {
		    if ( isSeparatorRequired ) {
			this.addSeparator();
			isSeparatorRequired = false;
		    }
		    this.add( MyMenuItem.create( child,
						 actions,
						 radiobuttonGroups,
						 resources,
						 canShowMnemonics ) );
		    isMenuEmpty = false;
		}
	    } else {
		throw new IllegalArgumentException();
	    }
	}
    }

    private static final String MENUS_PREFIX = "menus.";
}
