/*
 * MyMenuBar.java --- Build a menu bar from the XML description.
 *
 * Copyright (c) 2002, 2003, 2004 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import javax.swing.JMenuBar;
import javax.swing.UIManager;

import nanoxml.XMLElement;


public class MyMenuBar extends JMenuBar {
    public MyMenuBar( XMLElement xml, Map actions, Resources resources ) {
	super();

	Map	radiobuttonGroups = new HashMap();
	boolean isMac		  = System.getProperty( "mrj.version" ) != null;
	boolean isMacAqua	  = isMac
	    && UIManager.getSystemLookAndFeelClassName().equals(
		UIManager.getLookAndFeel().getClass().getName() );

	Enumeration childs = xml.enumerateChildren();
	while ( childs.hasMoreElements() ) {
	    XMLElement child = (XMLElement)childs.nextElement();
	    if ( child.getName().equals( "menu" ) ) {
		this.add( new MyMenu( child,
				      actions,
				      radiobuttonGroups,
				      resources,
				      !isMac,
				      isMacAqua ) );
	    } else {
		throw new IllegalArgumentException();
	    }
	}
    }

    static {
	// This property has the effect that the menu bar is on the top
	// of the screen instead of the top of the window on Mac OS.  It
	// has no effects on other systems.
	System.setProperty( "com.apple.macos.useScreenMenuBar",
			    Boolean.TRUE.toString() );
    }
}
