/*
 * MyMenuItem.java --- Build a menu from the XML description.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.StringTokenizer;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

import nanoxml.XMLElement;

public final class MyMenuItem {

    static JMenuItem create( XMLElement	xml,
			     Map	actions,
			     Map	radiobuttonGroups,
			     Resources	resources,
			     boolean	canShowMnemonics ) {

	String		type   = xml.getStringAttribute( "type", "?" ).trim();
	String		id     = xml.getStringAttribute( "name", "?" ).trim();
	Action		action = (Action)actions.get( id );
	JMenuItem	menuItem;
	if ( type.equals( "checkbox" ) ) {
	    menuItem = new MyJCheckBoxMenuItem( action );
	} else if ( type.equals( "radiobutton" ) ) {
	    String	group = xml.getStringAttribute( "group", "?" ).trim();
	    ButtonGroup bg    = (ButtonGroup)radiobuttonGroups.get( group );
	    if ( bg == null ) {
		bg = new ButtonGroup();
		radiobuttonGroups.put( group, bg );
	    }
	    menuItem = new MyJRadioButtonMenuItem( action, bg );
	} else {
	    menuItem = new JMenuItem( action );
	}

	if ( action == null ) {
	    menuItem.setEnabled( false );
	}

	try {
	    StringTokenizer t =
		new StringTokenizer(
		    resources.getString( MENUS_PREFIX + id ).trim(), "," );

	    NameAndMnemonic nam = new NameAndMnemonic( t.nextToken() );
	    menuItem.setText( nam.name() );
	    if ( canShowMnemonics && nam.hasMnemonic() ) {
		menuItem.setMnemonic( nam.mnemonic() );
	    }

	    if ( t.hasMoreTokens() ) {
		menuItem.setAccelerator(
		    KeyStroke.getKeyStroke( t.nextToken().trim().charAt( 0 ),
					    DEFAULT_KEY_MODIFIER ) );
	    }
	} catch ( MissingResourceException e ) {
	    menuItem.setText( id );
	}

	return menuItem;
    }

    private static class MyJCheckBoxMenuItem extends JCheckBoxMenuItem
	implements PropertyChangeListener {

	MyJCheckBoxMenuItem( Action action ) {
	    super( action );
	    if ( action != null ) {
		action.addPropertyChangeListener( this );
		setSelected(
		    ( (Boolean)action.getValue( SELECTED_PROPERTY_NAME ) )
		    .booleanValue() );
	    }
	}

	public void propertyChange( PropertyChangeEvent evt ) {
	    setSelected( ( (Boolean)evt.getNewValue() ).booleanValue() );
	}
    }

    private static class MyJRadioButtonMenuItem extends JRadioButtonMenuItem
	implements PropertyChangeListener {

	MyJRadioButtonMenuItem( Action action, ButtonGroup group ) {
	    super( action );
	    if ( action != null ) {
		action.addPropertyChangeListener( this );
		setSelected(
		    ( (Boolean)action.getValue( SELECTED_PROPERTY_NAME ) )
		    .booleanValue() );
	    }
	    group_ = group;
	    group.add( this );
	}

	public void propertyChange( PropertyChangeEvent evt ) {
	    if ( evt.getPropertyName().equals( SELECTED_PROPERTY_NAME ) ) {
		boolean b = ( (Boolean)evt.getNewValue() ).booleanValue();
		setSelected( b );
		if ( b ) {
		    Enumeration enum = group_.getElements();
		    while ( enum.hasMoreElements() ) {
			MyJRadioButtonMenuItem rb
			    = (MyJRadioButtonMenuItem)enum.nextElement();
			if ( rb != this ) {
			    rb.getAction().putValue( SELECTED_PROPERTY_NAME,
						     Boolean.FALSE );
			}
		    }
		}
	    }
	}

	//----------------------------------------------------------------------

	private ButtonGroup	group_;
    }

    //--------------------------------------------------------------------------

    private MyMenuItem() { } // Avoid instantiation

    //--------------------------------------------------------------------------

    private static final String MENUS_PREFIX	       = "menus.";
    private static final String SELECTED_PROPERTY_NAME = "selected";

    private static final int DEFAULT_KEY_MODIFIER
	= Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
}
