/*
 * MyProperties.java --- Extension of java.util.Properties class
 *
 * Copyright (c) 2001, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

public class MyProperties extends Properties {

    public MyProperties( String defaultFile, String file ) {
	this( defaultProperties( defaultFile ), file );
    }

    public MyProperties( Properties defaultProperties, String file ) {
	super( defaultProperties );

	try {
	    FileInputStream in = new FileInputStream( file );
	    this.load( in );

	    propertiesFilename = file;
	} catch ( IOException ioe ) {
	    // Nothing to do
	}

	listeners = new LinkedList();
    }

    public void store( String header )
	throws IOException {

	FileOutputStream out = new FileOutputStream( propertiesFilename );
	this.store( out, header );
    }

    public Object setProperty( String key, String value ) {
	Object o = super.setProperty( key, value );
	if ( !value.equals( o ) ) {
	    firePropertyChanged( key );
	}
	return o;
    }

    public Object setProperty( String key, boolean value ) {
	return setProperty( key,
			    value
			    ? Boolean.TRUE.toString()
			    : Boolean.FALSE.toString() );
    }

    public int getIntProperty( String key ) {
	return Integer.valueOf( this.getProperty( key ) ).intValue();
    }

    public boolean getBooleanProperty( String key ) {
	return Boolean.valueOf( this.getProperty( key ) ).booleanValue();
    }

    //--------------------------------------------------------------------------

    private static Properties defaultProperties( String fileName ) {
	Properties p = null;
	try {
	    InputStream	is
		= new MyProperties().getClass().getResourceAsStream( fileName );
	    p = new Properties();
	    p.load( is );
	} catch ( IOException ioe ) {
	    System.err.println( "Unable to load default properties file : "
				+ fileName );
	    System.exit( 0 );
	}
	return p;
    }

    private MyProperties() { }

    //--------------------------------------------------------------------------
    // Listener part

    private class Listener {
	Listener( MyPropertiesListener aListener, String aFilter ) {
	    listener = aListener;
	    filter   = aFilter;
	}
	private MyPropertiesListener	listener;
	private String			filter;
    }

    public void addPropertyListener( MyPropertiesListener listener ) {
	addPropertyListener( listener, "" );
    }

    public void addPropertyListener( MyPropertiesListener	listener,
				     String			filter ) {
	listeners.add( new Listener( listener, filter ) );
    }

    private void firePropertyChanged( String property ) {
	Iterator it = listeners.iterator();
	while ( it.hasNext() ) {
	    Listener l = (Listener)it.next();
	    if ( property.startsWith( l.filter ) ) {
		l.listener.propertyChanged( property );
	    }
	}
    }

    //--------------------------------------------------------------------------

    private String	propertiesFilename;
    private List	listeners;
}
