/*
 * NameAndMnemonic.java --- Format a menu name and extract the mnemonic.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import javax.swing.AbstractButton;



public class NameAndMnemonic {

    public NameAndMnemonic( String input ) {
	extractNameAndMnemonic( input );
    }

    public String name() {
	return this.name;
    }

    public boolean hasMnemonic() {
	return this.hasMnemonic;
    }

    public char mnemonic() {
	return this.mnemonic;
    }

    //--------------------------------------------------------------------------

    public static void setTextAndMnemonic( AbstractButton	button,
					   String		text ) {
	NameAndMnemonic nam = new NameAndMnemonic( text );
	button.setText( nam.name() );
	if ( nam.hasMnemonic() ) {
	    button.setMnemonic( nam.mnemonic() );
	}
    }

    //--------------------------------------------------------------------------

    private void extractNameAndMnemonic( String input ) {
	hasMnemonic = false;
	if ( input != null ) {
	    name = input.trim();

	    int lastIndex = 0;
	    int index = name.indexOf( MNEMONIC_INDICATOR );
	    while ( index != -1 ) {
		if ( index != name.length() - 1 ) {
		    if ( name.charAt( index + 1 ) != MNEMONIC_INDICATOR
			 && !hasMnemonic ) {
			hasMnemonic = true;
			mnemonic =
			    Character.toLowerCase( name.charAt( index + 1 ) );
		    }
		    name = name.substring( 0, index )
			+ name.substring( index + 1 );
		}
		index = name.indexOf( MNEMONIC_INDICATOR, index + 1 );
	    }
	} else {
	    name = null;
	}
    }

    //--------------------------------------------------------------------------

    private static final char MNEMONIC_INDICATOR = '&';

    private String	name;
    private boolean	hasMnemonic;
    private char	mnemonic;

}
