/*
 * NameAndMnemonicTests.java --- JUnit tests for class NameAndMnemonic.
 *
 * Copyright (c) 2002, 2003, 2004 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import javax.swing.AbstractButton;
import javax.swing.JButton;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class NameAndMnemonicTests extends TestCase {

    public NameAndMnemonicTests( String name ) {
	super( name );
    }

    public static Test suite() {
	return new TestSuite( NameAndMnemonicTests.class );
    }

    public static void main( String[] args ) {
	junit.textui.TestRunner.run ( suite() );
    }

    //-------------------------------------------------------------------------

    public void testAll() {

	NameAndMnemonic	nam = new NameAndMnemonic( " &Couper  " );
	assertEquals( "Couper", nam.name() );
	assertTrue( nam.hasMnemonic() );
	assertEquals( 'c', nam.mnemonic() );

	nam = new NameAndMnemonic( "Couper" );
	assertEquals( "Couper", nam.name() );
	assertTrue( !nam.hasMnemonic() );

	nam = new NameAndMnemonic( "Black && White" );
	assertEquals( "Black & White", nam.name() );
	assertTrue( !nam.hasMnemonic() );

	nam = new NameAndMnemonic( "Libertad&" );
	assertEquals( "Libertad&", nam.name() );
	assertTrue( !nam.hasMnemonic() );

	nam = new NameAndMnemonic( "&Black && White" );
	assertEquals( "Black & White", nam.name() );
	assertTrue( nam.hasMnemonic() );
	assertEquals( 'b', nam.mnemonic() );

	nam = new NameAndMnemonic( "Black && &White" );
	assertEquals( "Black & White", nam.name() );
	assertTrue( nam.hasMnemonic() );
	assertEquals( 'w', nam.mnemonic() );

	nam = new NameAndMnemonic( "&Black or &White" );
	assertEquals( "Black or White", nam.name() );
	assertTrue( nam.hasMnemonic() );
	assertEquals( 'b', nam.mnemonic() );
    }

    public void testSetTextAndMnemonicForButton() {
	AbstractButton b = new JButton();
	NameAndMnemonic.setTextAndMnemonic( b, "Black && &White" );
	assertEquals( "Black & White", b.getText() );
//	assertEquals( 'w', b.getMnemonic() );
    }

}
