/*
 * Resources.java --- Get resources strings depending on the languages.
 *
 * Copyright (c) 2000, 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.net.URL;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

public class Resources {

    public Resources( String resourcesName ) {
	try {
	    resourceBundle = ResourceBundle.getBundle( resourcesName,
						       Locale.getDefault() );
	} catch ( MissingResourceException mre ) {
	    System.err.println( resourcesName + ".properties not found" );
	    System.exit( 1 );
	}
    }

    public String getString( String s ) {
	return resourceBundle.getString( s );
    }

    public URL getResource( String key ) {
	try {
	    String name = getString( key );
	    return this.getClass().getResource( name );
	} catch ( MissingResourceException e ) {
	    return null;
	}
    }

    public String[] getTokenizedString( String input ) {
	String	str;
	try {
	    str = getString( input );
	} catch ( MissingResourceException e ) {
	    str = null;
	}

	StringTokenizer t   = new StringTokenizer( str );
	String[]	cmd = new String[ t.countTokens() ];
	int		i   = 0;

	while ( t.hasMoreTokens() ) {
	    cmd[ i ] = t.nextToken();
	    i++;
	}

	return cmd;
    }

    //--------------------------------------------------------------------------

    /**
     * Suffix applied to the key used in resource file
     * lookups for an image.
     */
    public static final String IMAGE_SUFFIX = "-Image";

    /**
     * Suffix applied to the key used in resource file
     * lookups for a label.
     */
    public static final String LABEL_SUFFIX = "-Label";

    /**
     * Suffix applied to the key used in resource file
     * lookups for tooltip text.
     */
    public static final String TIP_SUFFIX = "-Tooltip";

    //--------------------------------------------------------------------------

    private ResourceBundle	resourceBundle;

}
