/*
 * ToolBar.java --- Build a toolbar from the XML description.
 *
 * Copyright (c) 2002, 2003 by Pascal Wassong All Rights Reserved.
 *
 * This file is part of my personal tool box.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package wassong.gui.tools;

import java.awt.Component;
import java.awt.Insets;
import java.util.Map;
import java.util.MissingResourceException;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JToolBar;

import nanoxml.XMLElement;
import java.util.Enumeration;


public class ToolBar extends JToolBar {

    public ToolBar( String	title,
		    XMLElement	xml,
		    Map		actions,
		    Map		components,
		    Resources	resources ) {

	super( title );

	Enumeration childs = xml.enumerateChildren();
	while ( childs.hasMoreElements() ) {
	    XMLElement	child = (XMLElement)childs.nextElement();
	    String	name  = child.getName();
	    if ( name.equals( "menuitem" ) ) {
		add( createCommand( child, actions, components, resources ) );
	    } else if ( name.equals( "separator" ) ) {
		addSeparator();
	    } else {
		throw new IllegalArgumentException();
	    }
	}


	add( Box.createHorizontalGlue() );
    }

    //--------------------------------------------------------------------------

    private Component createCommand( XMLElement	xml,
				     Map	actions,
				     Map	components,
				     Resources	resources ) {

	String id = xml.getStringAttribute( "name", "?" ).trim();

	Object a = actions.get( id );
	Object c = components.get( id );
	if ( a != null ) {
	    return createToolbarButton( id, (Action)a, resources );
	} else if ( c != null ) {
	    return (Component)c;
	} else {
	    return createToolbarButton( id, null, resources );
	}
    }

    private JButton createToolbarButton( String		key,
					 Action		a,
					 Resources	resources ) {
	JButton b = new JButton() {
		public float getAlignmentY() { return 0.5f; }
	    };
	b.setRequestFocusEnabled( false );
	b.setMargin( new Insets( 1, 1, 1, 1 ) );

	if ( a != null ) {
	    b.setAction( a );
	    if ( a.getValue( Action.SMALL_ICON ) != null ) {
		b.setText( "" );
	    }
	} else {
	    b.setEnabled( false );
	}

	try {
	    b.setToolTipText( resources.getString(
				  key + resources.TIP_SUFFIX ) );
	} catch ( MissingResourceException e ) {
	    // Nothing to do
	}

	return b;
    }

}
